/**********************************************************************************************
  Burst.h   - Main header file for burst.exe

  3-17-09: The PHOTON_STREAM struct now has a 256-length char array for 
           the data's full-path filename.

           GetOpenFile() function prototype now has a PHOTON_STREAM struct 
           pointer as a parameter.  
 *********************************************************************************************/

#pragma once
#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <malloc.h>
#include "qcwin.h"
#include "menus.h"
#include "resource.h"
#include "PHCFile.h"
#include <time.h>    
#include <commctrl.h>
#include <shlobj.h>
#include <process.h>

//#define DEBUGMODE   1

// version info
#define BURST_VER              3
#define BURST_REV              0

// IO error messages
#define NO_ERRORS               0
#define FILE_OPEN_ERROR         1
#define MEM_ERROR               2
#define ZERO_BYTES_READ         3
#define USER_CANCEL             4
#define FREE_MEM_ERROR          5
#define NOT_A_PHCFILE           6

// Burst Analysis error codes are defined 
// in the string table in Burst.rc

// Counts window mouse function defines
#define FALL_THROUGH      0
#define FIND_BACKGROUND   1
#define SET_AC_LIMIT      2

// data types - used by hook.c for saving tab-delimited data sets
#define UNDEFINED_DATA   0
#define FCS_DATA         1
#define SEGFCS_DATA      2
#define FCCS_DATA        3
#define SEGFCCS_DATA     4
#define PCH_DATA         5
#define BURST_DATA       6
#define COBURST_DATA     7

// supported file types
#define UNSUPPORTED     0
#define PHC_FILE        1
#define CORCOM_FILE     2
#define BH_FILE         3
#define RAW_WORD        4
#define RAW_BYTE        5

// defines for indices of plotdata struct
#define BURSTSIZE               0
#define WIDTH                   1
#define INTERVAL                2

// defines for displaying counts
#define US_TO_SEC               1.0E-6
#define QC_SIZE                 1000            //  number of points for QC plots

// misc defines 
#define MAXBUR                  10000            // max number of bursts allowed
#define DIRMAX                  256              // max char's for dir buffer
#define FCS_FRACTION            0.50             // fraction of trace time to AC to.
#define LN2                     0.69314718056    // ln(2)

// marco defines
#define ARROW_CURSOR       SetCursor(LoadCursor (NULL, IDC_ARROW))
#define HOURGLASS_CURSOR   SetCursor(LoadCursor (NULL, IDC_WAIT))
#define GETPS(set)         (set == 0) ? &ps1 : &ps2
#define GETBURST(set)      (set == 0) ? &burst1 : &burst2
#define GETFCS(set)        (set == 0) ? &fcs1 : &fcs2
#define GETOTHER(set)      (set == 0) ? 1 : 0

/******************************************************************************
 Data structure for general parameters and user stored values (via ini file)
 *****************************************************************************/
typedef struct  tagINIT_PRM
{
  DWORD      TriggerCounts;       // level that triggers a "burst search"
  DWORD      MinCountsPerBurst;   // minimum counts that is a burst
  DWORD      SubMinBins;          // allowed "inbetween" bins
  DWORD      NumHistBins;         // number of histogram bins
  DWORD      BackGrdCounts;       // last bk count value
  double     BurstBinsize;        // width of burstsize bins in histogram plot
  double     WidthBinsize;        // width of burst width bins in histogram plot
  double     IntervalBinsize;     // width of interval bins in histogram plot
  double     f1;                  // bleedthrough correction fraction
  double     f2;                  // bleedthrough correction fraction
  double     rab;                 // bleedthrough correction - channel A/(A in B)
  double     rba;                 // bleedthrough correction - channel B/(B in A)
  double     a1;                  // multiplier for PS1
  double     a2;                  // multiplier for PS2
  double     c1;                  // constant 1 for PS1
  double     c2;                  // constant 2 for PS2
  BOOL       bBackgroundSubtract; // if true subtract a calculated background 
  BOOL       bAutoScale;          // autoscale flag
  BOOL       bHistAutoScale;      // autoscale flag for histogram scaling
  BOOL       bShowBurstMarkers;   // flag to toggle burst marker visibility 
  BOOL       bShowFCSDialog;      // display the FCS setup dialog box before doing autocorr
  BOOL       bShowHistDialog;     // display the histigram setup dialog box before burst analysis
  BOOL       bAppendBurstResults; // after saving an individual burst results file, append burst data to a master file 
  BOOL       bBatchFCS;           // if TRUE, calc FCS curve of each data set to batch processing mode
  BOOL       bBatchSegFCS;        // if TRUE, calc segFCS curve of each data set to batch processing mode
  BOOL       bBatchCorCom;        // if TRUE, batch analysis will look for Correlator.com dat files
  BOOL       bBatchFilter;        // if TRUE, filter data before busrt analysis
  BOOL       bBatchEstBkgd;       // if TRUE use background estimate algorthm to find background level
  int        SGFilterWidth;       // number of points to do the SG over
  WORD       SumStep;             // number of lags to calc before summing data (multitau)
  double     BinWidth;            // system binwidth in s
  double     PCHBinWidth;         // time to sum over for PCH analysis
  double     FCSBinWidth;         // binsize that regular and segmented AC's occur at
  double     SegMinTime;          // minimal segment interval to autocorrelate
  WORD       SegSumStep;          // sum step used for segmented autocorrelations
  int        Displaymode;         // flag for how counts are displayed - averaged, summed or single pt
} INIT_PRM; 

/***************************************************************************************
 Structure for raw data (photon counts)
 **************************************************************************************/
typedef struct tagPHOTON_STREAM
{
 char   szFilePath[256]; // Full path filename   (3-17-09)
 char   szFilename[256]; // filename 
 int    FileType;        // type of data file (PHC, dat, or other)
 int    PHCFiletype;     // if a PHC file,the compression mode and data size - defined in PHCFile.h
 int    Hardware;        // type of hardware that roginally collected the file
 int    SGFilterPasses;  // SG filter passes
 DWORD  nbins;           // number of data points (bins)
 DWORD  Bkg1;            // index of counts for bkg calc's 
 DWORD  Bkg2;            // index of counts for bkg calc's 
 DWORD  ACindex1;        // index of counts for AC and PCH calcs
 DWORD  ACindex2;        // index of counts for AC and PCH calcs
 WORD   max;             // max number of counts per bin
 WORD   min;             // min number of counts per bin
 double CPS;             // counts per sec          
 double BinWidth;        // binwidth is s (may differ from original binwidth)
 double BaseBinWidth;    // base or initial bin width of file (for Cor.com files = 1/MHz)
 double TraceLength;     // total trace time (BinWidth * nbins * /10^6, i.e. in sec)
 double NumPhotons;      // total number of photons in trace
 double BkMean;          // average backgroud or dark counts
 double BkSD;            // sd of mean
 LPWORD counts;          // ptr to virtual memory that holds count data
 float  *fcounts;        // ptr to virtual memory for count data and SG smoothing
} PHOTON_STREAM;


/***************************************************************************************
 Structure for burst data
 **************************************************************************************/
typedef struct tagBURST
{
  char      szFilename[256];     // filename of dataset that burst data is from 
  char      szAppendFile[256];   // filename of appended burst results data set
  DWORD     numbursts;           // number of bursts found
  double    BinWidth;            // binwidth in s
  DWORD     TriggerCounts;       // level that triggers a "burst search"
  DWORD     MinCountsPerBurst;   // minimum counts that is a burst
  DWORD     MaxBurstWidth;       // maximum time in s to look ahead for a burst
  DWORD     BackGrdCounts;       // background counts
  DWORD     SubMinBins;          // allowed "inbetween" bins >= BackGrdCounts when defining a burst 
  DWORD     dwAnalysisSize;      // size in bytes of burst data arrays
  DWORD     dwAnalysisPts;       // maximum number of bursts (dwAnalysisSize/8)
  double    BkMean;              // mean selected or measured BkGrd
  double    BkSD;                // selected or measured BkGrd sd 
  double    *Burst;              // number of photons in the burst (photons/burst)
  double    BurstMean;           // mean number of photons per burst
  double    BurstSD;             // SD of mean number of photons per burst
  double    BurstMax;            // max photons per burst
  double    BurstMin;            // min photons per burst
  double    *Width;              // burst width  (in s)
  double    WidthMean;           // mean burst width in s
  double    WidthSD;             // SD of mean burst width
  double    WidthMax;            // max burst width
  double    WidthMin;            // min burst width
  double    *Interval;           // intervals between bursts  (in ms) 
  double    IntervalMean;        // mean interval between bursts in sec
  double    IntervalSD;          // SD of mean interval between bursts in sec
  double    IntervalMax;         // max interval between bursts
  double    IntervalMin;         // min interval between bursts
  double    *Peak;               // peak number of photons/bin per burst
  double    PeakMean;            // mean peak number of photons per burst
  double    PeakSD;              // SD of mean peak number of photons per burst
  double    PeakMax;             // max value of peak number of photons/bin
  double    PeakMin;             // min value of peak number of photons/bin
  double    *Time;               // time of burst (in sec) defined as end time - (end time - start time)/2 
  DWORD     *Start;              // start index of burst
  DWORD     *End;                // end index index of burst
}  BURST; 


/*************************************************************************************
 Autocorrelation working data structure - holds the variables used to calculate
 the normalized autocorrelation.  This is part of the AUTOCORRELATION structure below
 and will be initialized in GetFCSMemory() or GetSFCSMemory(). 
 **************************************************************************************/
typedef struct tagAUTOCORR_DATA
{
  double Lag;      // lag time in ms 
  DWORD  n;        // number of bins at each lag
  __int64 Go;       // raw autocorrelation
  __int64 Mi;       // sum 1
  __int64 Mk;       // sum 2 (lag)
  double Gn;       // normalized AC
}  AUTOCORR_DATA ;

/*************************************************************************************
 Autocorrelation data structure - this is used for both single trace data
 AC and segmented AC.  start_index and end_index should be dim'd to 1 for a 
 single trace AC and nave set to 1.  
 **************************************************************************************/
typedef struct tagAUTOCORRELATION
{
 char           szFilename[256];     // base filename 
 int            DataType;            // FCS curve or FCCS curve
 DWORD          sumstep;             // summation step for AC
 double         binwidth;            // binwidth in s of data for autocorrelation (usually 1 s)
 double         burst_binwidth;      // binwidth in s of stream when used for burst analysis
 double         mintime;             // minimum interval time used (for segmented AC)
 double         TraceLength;         // total trace time (BinWidth * nbins * /10^6, i.e. in sec)
 double         maxinterval;         // time (in s) between start and end
 double         FcsMaxPow2;          // exponent of last lag term 
 double         segtimes[MAXBUR];    // length of segments in s
 DWORD          start_index[MAXBUR]; // start index of interval
 DWORD          end_index[MAXBUR];   // start index of interval
 BYTE           ValidSeg[MAXBUR];    // flag to mark valid segment based on length 
 DWORD          segmaxpts;           // number of points in maximum length segment
 DWORD          numseg;              // total number of segments between bursts
 DWORD          nave;                // number of valid intervals to average 
 DWORD          ntau;                // number of tau's based on max interval 
 DWORD          dwPtsToCorr;         // number of points being ac'd 
 DWORD          maxindex;            // index of max segment length
 AUTOCORR_DATA  *ac;                 // AUTOCORR structure 
}  AUTOCORRELATION;

/***************************************************************************************
 Structure for batch burst analysis data
 ***************************************************************************************/
typedef struct tagBATCH_BURST
{
  char szDir[MAX_PATH];       // working directory
  char szSaveFileName[128];   // base name for save file 
  int NumFiles;               // number of files in dir used in analysis
  char **szFileNames;         // array for file names of valid files
  BURST burst;                // burst data structure for total summed results
  PHOTON_STREAM ps;           // photon stream structure for data input 
  AUTOCORR_DATA *sac;         // segmented autocorrelation over all data sets
  DWORD  TriggerCounts;       // level that triggers a "burst search"
  DWORD  MinCountsPerBurst;   // minimum photons/burst that is a burst
  DWORD  SubMinBins;          // allowed "inbetween" bins
  DWORD  *BackGrdCounts;      // bkg count values
  int    SGFilterWidth;       // number of points to do the SG over, if 0, no smoothing
  double BinWidth;            // binwidth is s (may differ from original binwidth)
  BOOL bRun;                  // thread control flag
  BOOL bCompleted;            // signals that the analysis is complete and final data can be saved
  BOOL bDirSelected;          // if TRUE, a directory has been selected
  BOOL bAbort;                // abort button was clicked
  BOOL bBatchEstBkgd;         // if TRUE use background estimate algorthm to find background level
  int FileType;               // type of file in batch processing dir   - phc or dat (cor.com)
  HWND hDlg;                  // handle to dialog box 
}    BATCH_BURST;

/***************************************************************************************
 Photon Counting Histogram (PCH) data structure
 ***************************************************************************************/
typedef struct tagPC_HIST
{
  char           szFilename[128];   // filename from PS file
  DWORD          n;                 // number of histogram values
  double         *x;                // bin values
  double         *y;                // frequency 
  double         xmax;              // max photons/bin value
  double         ymax;              // max freq. value
}  PC_HIST ;

/***************************************************************************************
 Data/Plot  structure for creating histogram plots
 **************************************************************************************/
typedef struct  tagPLOTDATA
{
   PGRAPH_DEF    pGraph;         // graph handle
   HDATA         hData;          // data handle
   HGOBJ         hPlot;          // plot handle 
   HGOBJ         hAxisY;
   HGOBJ         hAxisX;
   HGOBJ         hTitle;         // title string handle
   BOOL          PlotType;       // line or bar
   int           n;              // number of pts
   DWORD         end;            // index of the last non-zero y plus 1
   double        *x;
   double        *y;
   double        xmin;
   double        ymin;
   double        xmax;
   double        ymax;
   double        xstep;
   double        ystep;
   char          szPlotTitle[128];
} PLOTDATA;

/*********************************************************************************
 Counts Data/Plot  structure
 ********************************************************************************/
typedef struct  tagCOUNTSPLOTDATA
{
   PGRAPH_DEF    pGraph;             // graph handle for photon stream
   int           n;                  // number of points plotted (QC_SIZE)
   HDATA         hData1;             // data handle for PS1
   HDATA         hData2;             // data handle for PS2
   HGOBJ         hPlot1;             // plot handle for PS1
   HGOBJ         hPlot2;             // plot handle for PS2
   HGOBJ         hAxisY;
   HGOBJ         hAxisX;
   HGOBJ         hCur1;              // cursors for the counts window to mark FCS start and stop
   HGOBJ         hCur2;             
   HGOBJ         hBurstMarker; 
   HGOBJ         hCurStart[MAXBUR];  // a bunch of handles for cursors used to mark the burst start and stop indices
   HGOBJ         hCurEnd[MAXBUR];
   int           Marker;             // index of current marker (arrow) position
   int           nMarkersInSegment;  // number of burst markers for the current segment (# of elements in above array)
   int           MarkerIndex[MAXBUR];// index of x array to position marker 
   int           BurstIndex[MAXBUR]; // index of bursts being displayed - needed for obtaining burst data for display
   DWORD         nbins;              // number of points in the total trace (note PS1 npts must = PS2 npts)
   DWORD         nPS;                // number of points represented by QC_SIZE long trace
   DWORD         cur1;               // indices for FCS region cursor - left
   DWORD         cur2;               // indices for FCS region cursor - right
   DWORD         end;                // index of the last non-zero y plus 1
   double        *x;                 // data arrays for display 
   double        *y1;                // data arrays for display 
   double        *y2;                // data arrays for display
   double        xmin;
   double        xmax;
   double        ymin;
   double        ymax;
   double        binwidth;           // current binwidth - copied from ps1 or ps2
   double        segtime;            // time in seconds being displayed
   double        TraceLength;        // total time of trace
   int           DataType;           // flag for FCS., PCH or other type of data - used by hook.c
   double        DisplayedFrac;      // fraction of data set displayed in the counts window
   int           Displaymode;        // average, sum or single pt
   DWORD         NumSegments;        // number of segments based on segment time 
   DWORD         SegNum;             // counter of segment being displayed
   DWORD         BinsPerSegPt;       // number of bins per segment point - total time = QC_SIZE * BinsPerSegPt * binwidth
   WORD          SegTimeTBMax;       // upper limit of segment time track bar - depends on nbins
   BOOL          bShowBurstMarkers;  // toggles showingthe QC cursors that mark the burst regions
} COUNTSPLOTDATA;

/*********************************************************************************
 Structure to hold various boolean flags
 ********************************************************************************/
typedef struct tagDATAFLAGS
{
  BOOL            bSet[2];               // data loaded flag for PS 1 and 2
  BOOL            bResultsReady[2];      // TRUE when burst results are present of PS1 or PS2
  BOOL            bResultsSaved[2];      // TRUE if burst results have been saved
  double          BinWidth;              // binwidth of the 1st loaded data set
}  DATAFLAGS;


/************************************************************************************
 Global variables 
 ************************************************************************************/
#define         NUMTEXT    8           // number of text objects for results display

HINSTANCE       hInst;                 // global instance
HWND            hPSControls;           // handle to photon stream view control
HACCEL          hAccel;                // handle to accelerator
RECT            mw;                    // main window (counts plot and main menu)
RECT            rw;                    // results window
PGRAPH_DEF      pText;                 // ptr to main QC window
HGOBJ           hText[NUMTEXT];        // QC text objects for info display
HGOBJ           hPCHPlot;         
HGOBJ           hDataTemp;
PPAGE_DEF       pResultsPage;          // handle to QC windo for results
INIT_PRM        init;                  // initialization prms
PHOTON_STREAM   ps1;                   // raw data structure for channel 1
PHOTON_STREAM   ps2;                   // raw data structure for channel 2
COUNTSPLOTDATA  cd;                    // counts plot 
PC_HIST         pch;                   // PCH data structure 
BURST           burst1;                // burst structure for channel 1
BURST           burst2;                // burst structure for channel 2
PLOTDATA        pd[3];                 // 3 structures to store burst histogram plotting info 
AUTOCORRELATION fcs1;                  // FCS calculation space
AUTOCORRELATION fcs2;                  // FCS calculation space
AUTOCORRELATION fccs;                  // cross-correlation calculation space
PLOTDATA        pdFCS;                 // plot data structure for the fsc curve
DATAFLAGS       flags;                 // global structure that carries the status of data arrays
int             active_ps;             // either 0 or 1 for photon stream1 or photon stream 2
char            szWindowTitle[256];    // global string buffer for title bar of main window
int             fCountsMouse;          // flag for count window mouse
char            szCurrentDir[DIRMAX];  // current dir for ini file
BOOL            bShowResultsWin;       // toggles showing result plot window


/******************************************************************************************************
    FUNCTION PROTOTYPES 
 ***************************************************************************************************/
// Burst.c prototypes 
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
void   InitGlobals(void); 
BOOL   CheckScreenSize(void);
void   SetupMainWindow(HWND hwnd);
void   GreyOutBurstButtons(BOOL bState);
void   SetActivePS(int set);	  
void   CloseDataSet(int set);
void   ToggleResultsWindow(HWND hwnd);
void   ShowDataTable(int set);
void   BurstAnalysis(HWND hwnd, int set);
void   LoadDataFile(HWND hwnd, int set);
void   CrossCorrelate(HWND hwnd, int set);
void   DoPSMath(int command);
void   SegmentedAutocorrelation(HWND hwnd, int set);
void   HelpSystemDialog(HWND hwnd) ;
void   UpdateCPSEditbox(void);
void   DisplayPCHPlot(HWND hwnd, int set);
void   CountsPositionTrackBar(void);
void   SegmentTimeTrackBar(void);
void   SetDisplayedTime(void);
void   SetUpBinWidths(HWND hwnd, double binwidth);
void   GreyOutControls(BOOL bState);
void   SetupSegmentTimeTrackBar(void);
double GetBinWidth(HWND hwnd);
void   ToggleCountsAutoScale(void);
void   StepPositionTB(int direction);
void   ToggleBurstMarkerFlag(void);
void   ToggleHistAutoScale(void);
void   MoveBurstMarker(int direction, int set);
void   GetUserSetParameters(void);
int    GetPSIndex(int index, PHOTON_STREAM *oldps, PHOTON_STREAM *newps);
void   SetSGFilterWidths(HWND hwnd, int SGWidth);
int    GetSGFilterWidth(HWND hwnd);
int    StartBurstAnalysis(HWND hwnd, PHOTON_STREAM *ps, BURST *burst, int set); 
char   *GetString(char *szStr, UINT uID);

// BurstBatch.c prototypes
BOOL   CALLBACK BurstBatchProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
int    CALLBACK  BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData);
char   *GetDirectoryFromBrowse(char *szReturnedDir) ;
void   GetFilesForBatchProcessing(HWND hDlg, BATCH_BURST *bb);
void   BatchProcessThread(PVOID pvoid);
char   **StringArray(int NumStrings, int NumChar);
void   FreeStringArray(char **mptr, int NumStrings);
char   **FindFiles(char *szFileType, int *num, int NumChar) ;
void   SaveSummedBurstData(BURST *dest, BURST *source, char *szBase);
void   FreeBatchBurstStruct(BATCH_BURST *bb); 
char   *GetLastDirFromPath(char *szPath);
void   FreeBatchBurstStruct(BATCH_BURST *bb); 
void   SaveBatchBurstInfoFile(BATCH_BURST *bb); 
void   UpdateBatchStatus(HWND hDlg, char *szStr); 

// BurstCalc.c prototypes
BOOL   AnalyzeData(PHOTON_STREAM *ps, BURST *burst);
DWORD  LocateBurst(DWORD trigger_index, PHOTON_STREAM *ps, BURST *burst);
void   CalcMean(double *lpData, DWORD start, DWORD end, double *dMean, double *dDev, double *dMin, double *dMax); 
WORD   GetMaxWord(WORD *a, DWORD n, DWORD *NumCounts,  WORD *MinValue);
DWORD  FindPeak(WORD *a, DWORD start, DWORD end);
void   GetMinMax(double *a, DWORD n, double *dMin, double *dMax);
void   ReBinData(PHOTON_STREAM *ps, double NewBinWidth);
DWORD  NumberBinsAbove(DWORD dwVal, LPWORD a, DWORD n);
void   MultiplyDataSetByScalar(PHOTON_STREAM *ps, double value);
void   AddDataSets(PHOTON_STREAM *ps1, PHOTON_STREAM *ps2);
void   SubtractDataSets(PHOTON_STREAM *ps1, PHOTON_STREAM *ps2);
void   BleedThruCorrectDataSets(PHOTON_STREAM *ps1, PHOTON_STREAM *ps2, double frac1, double frac2);
void   SubtractConstantFromDataSet(PHOTON_STREAM *ps, double value);
void   AddPoisNoiseToDataSet(PHOTON_STREAM *ps);
void   AddConstantToDataSet(PHOTON_STREAM *ps, double value);
void   CalcPhotonCountHistogram(PHOTON_STREAM *ps, PC_HIST *pch, DWORD start, DWORD end);
void   EstimateBackground(PHOTON_STREAM *ps); 

//  BurstCorCom.c prototypes 
int    LoadCorrComFile(char *szFilename, PHOTON_STREAM *ps, double binwidth);

//  BurstFCS.c prototypes 
void   AutoCorrelate(HWND hwnd, int set); 
double AutoCorrelateMultiTau(PHOTON_STREAM *ps, AUTOCORRELATION *fcs, DWORD start, DWORD end);
double CrossCorrelateMultiTau(PHOTON_STREAM *ps1, PHOTON_STREAM *ps2, AUTOCORRELATION *fcs, DWORD start, DWORD end);
void   ReSegmentAnalysis(BURST *burst, AUTOCORRELATION *fcs);
BOOL   SegmentAnalysis(BURST *burst, AUTOCORRELATION *fcs, DWORD numbins);
int    SegmentedAutoCorrelateMultiTau(PHOTON_STREAM *ps, AUTOCORRELATION *fcs);
BOOL   AutocorrelateSegments(AUTOCORRELATION *fcs, int set);

//  BurstIO.c prototypes 
void  OpenDataFile(HWND hwnd, PHOTON_STREAM *ps, double binwidth, int index) ;
int   LoadPSWithNewBinWidth(char *szFilename, PHOTON_STREAM *ps, double binwidth) ;
int   GetPhotonCountFileType(char *szFilename);
int   LoadInitFile(void);
int   SaveInitFile(void);
int   SavePCHHist(char *szFilename, PC_HIST *pch) ;
void  SaveFCSCurve(HWND hwnd, int set) ;
int   SaveHistResults(HWND hwnd, int set); 
BOOL  SaveBurstResults(HWND hwnd, int set) ;
int   SaveDataSetAsPHCFile(HWND hwnd, int set);
void  SaveSegment(HWND hwnd, int set) ;
void  CreateIniFile(void);
char  *remove_fext(char *str);
char  *CreateDateFilename(char *fext);
char  *GetOpenFile (HWND hwnd, char * szFilter, PHOTON_STREAM *ps);   // 3-17-09
char  *GetSaveFile(HWND hwnd, char *szFilename, char *szFilter, char *szExt);
void  InitCountDisplayStruct(PHOTON_STREAM *ps, int set);

// BurstPHCFile.c prototypes 
int   LoadPHCFile(HWND hwnd, char *szFileName, PHOTON_STREAM *ps, double binwidth);
int   CompressPSDataByte(PHOTON_STREAM *ps, BYTE *DataType, DWORD *dwSize, LPBYTE bdata);
int   CompressPSDataWord(PHOTON_STREAM *ps, BYTE *DataType, DWORD *dwSize, LPWORD wdata); 
int   SavePHCFile(HWND hwnd, PHOTON_STREAM *ps, BOOL bCompress);
int   LoadCompressedBytePHC(PHOTON_STREAM *ps, PHCFILE_HEADER *fhd, LPBYTE lpByte);
int   LoadCompressedWordPHC(PHOTON_STREAM *ps, PHCFILE_HEADER *fhd, LPBYTE lpByte);
int   LoadUnCompressedBytePHC(PHOTON_STREAM *ps, PHCFILE_HEADER *fhd, LPBYTE lpByte);
int   LoadUnCompressedWordPHC(PHOTON_STREAM *ps, PHCFILE_HEADER *fhd, LPBYTE lpByte);

// BurstMem.c prototypes
LPVOID GetVirtualMem(DWORD dwSize); 
int    FreePhotonStreamMem(PHOTON_STREAM *ps);
BOOL   GetCorrelationMemory(AUTOCORRELATION *fcs);
void   FreeCorrelationMemory(AUTOCORRELATION *fcs);
BOOL   GetPCHMemory(PHOTON_STREAM *ps, PC_HIST *pch);
void   FreePCHMemory(PC_HIST *pch);
BOOL   GetBurstMem(BURST *burst, PHOTON_STREAM *ps);
void   FreeBurstMem(BURST *burst) ;
void   DisplayErrorMessage(void);
void   VirtualMemError(char *szStr);
void   GetCorrelationParameters(AUTOCORRELATION *fcs, DWORD start, DWORD end, WORD SumStep);

// BurstNoise.c prototypes
double RanNum(void);
double ExpDev(void);
double GamDev(int ia);
double PoisDev(double xm);
double GaussDev(void);
BOOL   FlipCoin(void);

// BurstPlot.c prototypes
void  CALLBACK InitCountsPage(PPAGE_DEF pPageDesc);
void  CALLBACK InitCountsGraph(PGRAPH_DEF pGrDesc, HDC hdc);
void  DisplayData(void);
void  CALLBACK InitResultsPage(PPAGE_DEF pPageDesc);
void  CALLBACK InitResultGraphs(PPAGE_DEF pPageDesc);
void  CALLBACK InitPhotonsGraph(PGRAPH_DEF pGrDesc, HDC hdc);
void  CALLBACK InitBurstWidthGraph(PGRAPH_DEF pGrDesc, HDC hdc);
void  CALLBACK InitIntervalGraph(PGRAPH_DEF pGrDesc, HDC hdc);
void  CALLBACK InitFCSGraph(PGRAPH_DEF pGrDesc, HDC hdc);
void  CALLBACK InitDrawText(PGRAPH_DEF pGrDesc, HDC hdc);
void  CalcHistogram(double *x, double *y, DWORD n, double *lpSort, DWORD start, DWORD end);
void  UpDateText(PHOTON_STREAM *ps, BURST *burst) ;
void  DisplayBkGrdText(void);
void  DisplayAlgorithmParms(void);
void  PlotHistogram(int win);
void  ClearGraphs(void);
void  ScaleHistArray(int win, BURST *burst); 
void  CALLBACK InitPCHGraph(PPAGE_DEF pPageDesc);
void  CALLBACK InitPCHPlot(PGRAPH_DEF pGrDesc, HDC hdc);
void  PlotFCSCurve(AUTOCORRELATION *fcs, char *szText);
DWORD GetPSIndexFromCountsIndex(int CountsXindex); 
DWORD GetCountsIndexFromPSIndex(int PSXindex) ;
void  MarkBursts(int set);
void  DisplayBurstHistograms(BURST *burst); 

// BurstProc.c protoypes
BOOL  CALLBACK BinDataProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL  CALLBACK AboutProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL  CALLBACK SegmentedAutocorrelationProc(HWND hDlg, UINT message, WPARAM wParam,  LPARAM lParam);
BOOL  CALLBACK CountsSetupProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL  CALLBACK HistSetupProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL  CALLBACK FCSSetupProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
void  DetectorCorrection(HWND hwndPB);
int   GetIndex(double nValue, double *nBuf, int n);
int   GetIndexWord(WORD nValue, WORD *nBuf, int n);

// Hook.c prototypes
void  TurnOffBackGroundSelect(HWND hwnd);  
void  TurnOffFCSSegmentSelect(HWND hwnd); 
DWORD GetCurIndex(double *x, double value, DWORD n, DWORD default_ret);   
void  DisplayBkGrdValue(DWORD x1, DWORD x2);
void  SaveACLimits(DWORD x1, DWORD x2);
void  CalcPSBkg(PHOTON_STREAM *ps);

// BurstSGFilter.c prototypes
BOOL SavitzkyGolaySmooth(float *dataset, int numdat, int numcoef, float *smoothdata, BOOL bZeroArray);
void SGSmoothData(int set);
BOOL SmoothPS(PHOTON_STREAM *ps); 




