/*****************************************************************************************************
  BurstIO.c   - IO routines for BURST.exe

  3-17-09: In OpenDataFile() func, the PHOTON_STREAM struct pointer is now  
           passed to the GetOpenFile() function call.

           In OpenDataFile() func, the PHOTON_STREAM struct's full-path filename
           is now copied to the local variable 'szFilename'.

           Function GetOpenFile() now has a PHOTON_STREAM struct pointer
           as a function parameter.

           In GetOpenFile(), both the base and full-path filenames are now
           extracted and stored in the PHOTON_STREAM struct.
/****************************************************************************************************/
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "resource.h"
#include "burst.h"

/*******************************************************************************************************************
  Called after a data set is loaded to set up the counts and flags structures
 ******************************************************************************************************************/
void InitCountDisplayStruct(PHOTON_STREAM *ps, int set)
{

 if(flags.bSet[GETOTHER(set)] && (cd.nbins < ps->nbins)) 
	 ps->nbins = cd.nbins; 
 else
	 cd.nbins = ps->nbins;

 ps->ACindex1 = 0;
 ps->ACindex2 = ps->nbins - 1;
 flags.bSet[set] = TRUE;
 flags.bResultsReady[set] = FALSE;
 flags.bResultsSaved[set] = FALSE;
 SetActivePS(set);
 cd.binwidth = ps->BinWidth;
 cd.TraceLength = cd.binwidth * cd.nbins * US_TO_SEC; 
 SetupSegmentTimeTrackBar();    // rescale the segment time track bar based on number of points
 SegmentTimeTrackBar();         
 UpdateCPSEditbox();
 SetDisplayedTime();
 GreyOutControls(FALSE);
 cd.bShowBurstMarkers = init.bShowBurstMarkers; 
 DisplayData();
}


/*******************************************************************************************************
  Opens a Counter card data file.   Be sure to set bSet[] in calling function
 ******************************************************************************************************/
void OpenDataFile(HWND hwnd, PHOTON_STREAM *ps, double binwidth, int set) 
{
  int  result;
  char szFilename[256];
  char szFilter[] = "Cor.Com photon stream file (*.dat)\0*.dat\0Photon count file (*.phc)\0*.phc\0";
  // "Photon count file (*.phc)\0*.phc\0Cor.Com photon stream file (*.dat)\0*.dat\0Becker-HicklH photon stream file (*.bhd)\0*.bhd\0";
  
  // get filename
  szFilename[0] = '\0';
  GetOpenFile(hwnd, szFilter, ps);      // 3-17-09
  strcpy(szFilename, ps->szFilePath);   // 3-17-09
  if(szFilename[0] == '\0') return;      
  
  // Get file type
  result = GetPhotonCountFileType(szFilename);
  if(result == UNSUPPORTED) {
	 MessageBox(hwnd, "Error opening file.", "File I/O Error", MB_OK);
	 return; 
  }
 
  // load  data 
  HOURGLASS_CURSOR;
  if(ps->counts != NULL) {        // clear existing data if it exists 
  	 FreePhotonStreamMem(ps);
  }
  
  ps->FileType = result;
  switch(ps->FileType)  {
     case CORCOM_FILE: 
          result = LoadCorrComFile(szFilename, ps, binwidth); 
		  break;
	 
	 case PHC_FILE:
		  result = LoadPHCFile(hwnd, szFilename, ps, binwidth); 
		  break;
  	 
	 case BH_FILE:
          //result = LoadBHFile(hwnd, szFilename, ps); 
		  break;

  }

  if(result == MEM_ERROR) {					         
     MessageBox(hwnd, "Could not reserve needed memory.", "Memory Error", MB_OK);
	 return;
  }

  if(ps->nbins < QC_SIZE) {					         
     MessageBox(hwnd, "Too few data points at requested bin size.", "Setup error", MB_OK);
	 return;
  }

 // if this is a normal data load - i.e. not a rebinning for FCS 
//  this func should be called to load the cd structure
 if(init.BinWidth == binwidth) InitCountDisplayStruct(ps, set);   

 ARROW_CURSOR;
}

/****************************************************************************
 Called by auto and cross correlate - reload a PS with the FCS binwidth
 ***********************************************************************/
int LoadPSWithNewBinWidth(char *szFilename, PHOTON_STREAM *ps, double binwidth) 
{
  int result; 

  // Get file type
  result = GetPhotonCountFileType(szFilename);
  if(result == UNSUPPORTED) {
	 MessageBox(NULL, "Error opening file.", "File I/O Error", MB_OK);
	 return 1; 
  }
  else 
	  ps->FileType = result;

  switch(ps->FileType)  {
     case CORCOM_FILE: 
          result = LoadCorrComFile(szFilename, ps, binwidth); 
		  if(result) return(result); 
		  break;
	 
	 case PHC_FILE:
		  result = LoadPHCFile(NULL, szFilename, ps, binwidth); 
  		  if(result) return(result); 
		  break;
  	 
	 case BH_FILE:
          //result = LoadBHFile(hwnd, szFilename, ps); 
 		  if(result) return(result); 
		  break;

  }
 return(NO_ERRORS); 
}


/********************************************************************************
 Returns a code to indicate type of photon count file. 
 Used by OpenDataFile()
 *******************************************************************************/
int GetPhotonCountFileType(char *szFilename)
{
  char szBuf[10];
  FILE *fp;

  // open file and see what type it is 
  fp = fopen(szFilename, "rb");
  if(fp == NULL) return(UNSUPPORTED);
  fread(szBuf, sizeof(char), 3, fp);
  fclose(fp);
  if(szBuf[0] == 'P' && szBuf[1] == 'H' && szBuf[2] == 'C') 
	  return(PHC_FILE);
  else if((szBuf[0] == 8 || szBuf[0] == 16) && (szBuf[1] == 40 || szBuf[1] == 60)) 
	  return(CORCOM_FILE);
  return(UNSUPPORTED); 
}



BOOL LoadInitFile(void)
{
  FILE *fp;
 
  GetCurrentDirectory(DIRMAX, szCurrentDir); 
  if((fp = fopen("burst.ini", "rb"))  == NULL) return(FALSE);
  fread(&init, sizeof(INIT_PRM), 1, fp);
  fclose(fp);
  return(TRUE);
}

BOOL SaveInitFile(void)
{
  FILE *fp;

  SetCurrentDirectory(szCurrentDir); 
  if((fp = fopen("burst.ini", "wb"))  == NULL) return(FALSE);
  fwrite(&init, sizeof(INIT_PRM), 1, fp);
  fclose(fp);
  return(TRUE);
}


BOOL SavePCHHist(char *szFilename, PC_HIST *pch) 
{
 DWORD i, n;
 FILE *fp;

 n = (DWORD) pch->n;
	 
 fp = fopen(szFilename, "wt");
 if (fp == NULL) {
       MessageBox(NULL, TEXT ("File cannot be opened"), TEXT ("FILE ERROR"), MB_OK);
	   return(TRUE);
 }
 for(i=0;i<n;i++)  fprintf(fp, "%f\t%f\n",pch->x[i], pch->y[i]);
 fclose(fp);
 return(FALSE);
}



/****************************************************************************************************
 Saves correlation curve 
 **************************************************************************************************/
void SaveFCSCurve(HWND hwnd, int set) 
{
 DWORD i;
 int NumHeaderRows;
 char szFilename[256];
 char szFilter[] = TEXT ("Text Data Files (*.txt)\0*.txt\0");
 char szExt[] = TEXT ("txt");
 FILE *fp; 
 AUTOCORRELATION *fcs = GETFCS(set);

 if(fcs->ntau == 0) return; 

 strcpy(szFilename, fcs->szFilename);
 remove_fext(szFilename);
 switch(fcs->DataType) {
      case FCS_DATA:  strcat(szFilename, "_fcs.txt"); break;
	  case FCCS_DATA: strcat(szFilename, "_fccs.txt"); break;
	  case SEGFCS_DATA: strcat(szFilename, "_segfcs.txt"); break;
	  case SEGFCCS_DATA: strcat(szFilename, "_segfccs.txt"); break;
 }

 strcpy(szFilename, GetSaveFile(hwnd, szFilename, szFilter, szExt));
 if(szFilename[0] == '\0') return;

 fp = fopen(szFilename, "wt");
 if (fp == NULL) {
       MessageBox(NULL, TEXT ("File cannot be opened"), TEXT ("FILE ERROR"), MB_OK);
	   return;
 }
 
 // write header
 switch(fcs->DataType) {
      case FCS_DATA:  
	  case FCCS_DATA: 
		   NumHeaderRows = 4;
           fprintf(fp, "Header size: \t%d rows\n", NumHeaderRows);  
           fprintf(fp, "PS File: \t%s\n", fcs->szFilename);  
           fprintf(fp, "MT summation step: \t%u\n", fcs->sumstep);  
           fprintf(fp, "Trace length: \t%g sec\n", fcs->maxinterval);  
		   break;
	  case SEGFCS_DATA: 
	  case SEGFCCS_DATA: 
		   NumHeaderRows = 6;
           fprintf(fp, "Header size: \t%d rows\n", NumHeaderRows);  
           fprintf(fp, "PS File: \t%s\n", fcs->szFilename);  
           fprintf(fp, "MT summation step: \t%u\n", fcs->sumstep);  
           fprintf(fp, "Trace length: \t%g sec\n", fcs->maxinterval); 
           fprintf(fp, "# segments averaged: \t%d\n", fcs->nave); 
           fprintf(fp, "Minimum seg. time: \t%g s\n", fcs->mintime); 
           break;
 }
 
 // write data
 for(i=0;i<fcs->ntau;i++) 
	 fprintf(fp, "%f\t%f\n",fcs->ac[i].Lag, fcs->ac[i].Gn);
	 //fprintf(fp, "%f\t%f\t%f\t%10.0f\t%10.0f\t%u\n",ac[i].Lag, ac[i].Gn, ac[i].Go, ac[i].Mi, ac[i].Mk, ac[i].n);
 fclose(fp);
 return;
}

/****************************************************************************************************
 Saves the histogrammed data
 **************************************************************************************************/
BOOL SaveHistResults(HWND hwnd, int set) 
{
 int i, dwMax;
 char szBlank[] = "--";
 char szFilter[] = TEXT ("Tab-delimited text files (*.txt)\0*.txt\0");
 char szExt[] = TEXT ("txt");
 char szFilename[256];
 PHOTON_STREAM *ps;

 FILE   *fp;
 

 ps = (set==0)? &ps1: &ps2; 
 strcpy(szFilename, ps->szFilename);
 remove_fext(szFilename);
 strcat(szFilename, "_hist.txt");
 strcpy(szFilename, GetSaveFile(hwnd, szFilename, szFilter, szExt));
 if(szFilename[0] == '\0') return(TRUE);
 
 fp = fopen(szFilename, "wt");
 if (fp == NULL) {
       MessageBox(hwnd, TEXT ("File cannot be opened"), TEXT ("FILE ERROR"), MB_OK);
	   return(1);
 }
  
 if(pd[0].n > pd[1].n) 
	  dwMax = pd[0].n;
 else 
	  dwMax = pd[1].n;

 if(pd[2].n > dwMax) dwMax = pd[2].n;

 fprintf(fp,"%s\t\t%s\t\t%s\n", "hv/burst", "width", "interval");
 for(i=0;i<dwMax;i++) {
	 if(i < pd[0].n) 
		 fprintf(fp, "%g\t%7.0f\t", pd[0].x[i], pd[0].y[i]);
	 else
         fprintf(fp, "%s\t%s\t", szBlank, szBlank);
	
     if(i < pd[1].n) 
		 fprintf(fp, "%g\t%7.0f\t", pd[1].x[i], pd[1].y[i]);
	 else
         fprintf(fp, "%s\t%s\t", szBlank, szBlank);
	 
	 if(i < pd[2].n) 
		 fprintf(fp, "%g\t%7.0f\t", pd[2].x[i], pd[2].y[i]);
	 else
         fprintf(fp, "%s\t%s\t", szBlank, szBlank);
	 fprintf(fp,"\n");
 }
 
 fclose(fp); 
 flags.bResultsSaved[set] = TRUE;
 return(FALSE);
}



/******************************************************************************************************
  Saves a list of bursts and the associated info
 ******************************************************************************************************/
BOOL SaveBurstResults(HWND hwnd, int set) 
{
 DWORD i, n;
 int NumHeaderRows;
 char szBlank[] = "--";
 char szFilter[] = TEXT ("Tab-delimited text files (*.txt)\0*.txt\0");
 char szExt[] = TEXT ("txt");
 char szFilename[256];
 FILE   *fp;
 BURST *burst = GETBURST(set);

 strcpy(szFilename, burst->szFilename);
 remove_fext(szFilename);
 strcat(szFilename, "_bursts.txt");
 strcpy(szFilename, GetSaveFile(hwnd, szFilename, szFilter, szExt));
 if(szFilename[0] == '\0') return(TRUE);
 
 fp = fopen(szFilename, "wt");
 if (fp == NULL) {
       MessageBox(NULL, TEXT ("File cannot be opened"), TEXT ("FILE ERROR"), MB_OK);
	   return(1);
 }
 
 // write header
 NumHeaderRows = 12;
 fprintf(fp, "Header size:\t%d rows\n", NumHeaderRows);  
 fprintf(fp, "PS File:\t%s\n", burst->szFilename);  
 fprintf(fp, "Number of bursts:\t%u\n",burst->numbursts );  
 fprintf(fp, "Binwidth:\t%4.1f s\n", burst->BinWidth);  
 fprintf(fp, "Background:\t%u\n", burst->BackGrdCounts);  
 fprintf(fp, "Triggerlevel:\t%u\n", burst->TriggerCounts);  
 fprintf(fp, "Min. photons/burst:\t%u\n", burst->MinCountsPerBurst); 
 fprintf(fp, "Allowed bins below bkgrd:\t%u\n", burst->SubMinBins);
 fprintf(fp, "Mean photons/burst:\t%u%u\n", (DWORD) burst->BurstMean, (DWORD) burst->BurstSD);  
 fprintf(fp, "Mean peak photons:\t%u%u\n", (DWORD) burst->PeakMean, (DWORD) burst->PeakSD);  
 fprintf(fp, "Mean burst width:\t%u%u s\n", (DWORD) burst->WidthMean, (DWORD) burst->WidthSD);  
 fprintf(fp, "Mean interval:\t%4.2f%4.2f sec\n", burst->IntervalMean, burst->IntervalSD);  
 
 // write data
 n = burst->numbursts;
 fprintf(fp,"%s\t%s\t%s\t%s\t%s\n", "time(s)", "hv/burst", "peak", "width(s)", "interval(s)");
 for(i=0; i<n; i++) {
	 fprintf(fp, "%5.6f\t%8.0f\t%8.0f\t%8.0f\t%8.6f\n", burst->Time[i], burst->Burst[i], burst->Peak[i], 
		           burst->Width[i], burst->Interval[i]);
 }
 fclose(fp); 
 flags.bResultsSaved[set] = TRUE;
 
 //if(flags
 
 
 
 return(FALSE);
}

/******************************************************************************************************
  Saves the current set of count data displayed in the count plot window
 ******************************************************************************************************/
void SaveSegment(HWND hwnd, int set) 
{
 int i;
 char szFilter[] = TEXT ("Text Data Files (*.txt)\0*.txt\0");
 char szExt[] = TEXT ("txt");
 char szFilename[256];
 PHOTON_STREAM *ps; 
 FILE   *fp;
 
 if(!flags.bSet[set]) {
	 MessageBox(hwnd, "No data in photon stream data selected as active PS.", "Data save error", MB_OK);
	 return;   // no data
 }

 ps = (set == 0)? &ps1 : &ps2;
 strcpy(szFilename, ps->szFilename); 
 remove_fext(szFilename);
 cd.SegNum;
 sprintf(szFilename, "%s_%u.txt", szFilename, cd.SegNum);
 strcpy(szFilename, GetSaveFile(hwnd, szFilename, szFilter, szExt));
 if(szFilename[0] == '\0') return; 
 fp = fopen(szFilename, "wt");
 if (fp == NULL) {
    MessageBox(hwnd, "File cannot be opened", "File open error", MB_OK);
    return;
 }
 
 for(i=0;i<cd.n;i++) fprintf(fp, "%g\t%7.0f\t%7.0f\n", cd.x[i], cd.y1[i], cd.y2[i]);
 fclose(fp); 
}

/*****************************************************************************************
  Sets initial values - called if burst.ini isn't found
 ***************************************************************************************/
void CreateIniFile(void)
{
 FILE *fp;
  
 init.TriggerCounts = 5;        // level that triggers a "burst search"
 init.MinCountsPerBurst = 10;   // minimum counts that is a burst
 init.SubMinBins = 10;          // allowed "inbetween" bins
 init.NumHistBins = 100;        // number of histogram bins
 init.BackGrdCounts = 1;        // last bk count value
 init.BurstBinsize = 0.0;       // binsize (if zero - use min photons/bin)
 init.WidthBinsize = 0.0;       // binsize (if zero - use min width)
 init.IntervalBinsize = 0.0;    // binsize (if zero - use min interval)
 init.f1 = 1.0;                 // bleedthrough correction constant
 init.f2 = 1.0;                 // bleedthrough correction constant
 init.rab = 0.5;                // bleedthrough correction - channel A/(A in B)
 init.rba = 0.5;                // bleedthrough correction - channel B/(B in A)
 init.a1 = 1.0;                 // multiplier of PS1
 init.a2 = 1.0;                 // multiplier of PS2
 init.c1 = 0.0;                 // constant 1
 init.c2 = 0.0;                 // constant 2
 init.bBackgroundSubtract = 0;  // if true subtract background from bursts
 init.bAutoScale = 0;           // if true autoscale counts plot
 init.bHistAutoScale = 1;       // if true autoscale histogram 
 init.bShowBurstMarkers = 1;    // show burst markers after analysis
 init.bShowFCSDialog = 1;       // display the FCS setup dialog box before doing autocorr
 init.bShowHistDialog = 1;      // display the histigram setup dialog box before burst analysis
 init.bAppendBurstResults = 1;  // save bursts to an appended file as well as individual files
 init.SGFilterWidth = 5;        // SG filter over 7 data points
 init.SumStep = 8;              // number of lags to calc before summing data
 init.BinWidth = 1.0;           // system binwidth in s
 init.PCHBinWidth = 10;         // time to sum over for PCH analysis
 init.FCSBinWidth = 1.0;        // biwidth for segmented autocorrelation
 init.SegMinTime = 100.0;       // minimal segment interval to autocorrelate
 init.SegSumStep = 8;           // sum step used for segmented autocorrelations
 init.Displaymode = IDC_CD_SINGLE; // flag for how counts are displayed - averaged, summed or single pt
 init.bBatchFCS = TRUE;
 init.bBatchSegFCS = TRUE;
 init.bBatchCorCom = TRUE; 
 init.bBatchEstBkgd = TRUE; 
 init.bBatchFilter = TRUE; 

 fp = fopen("burst.ini", "wb");
 fwrite(&init, 1, sizeof(INIT_PRM), fp);
 fclose(fp);
}

/****************************************************************************************************
  Saves a photon count data as a PHC file with the current binwidth
 **************************************************************************************************/
int SaveDataSetAsPHCFile(HWND hwnd, int set)
{
 PHOTON_STREAM *ps = GETPS(set);
 char szBuf[128];
 int result; 

 if(ps->BaseBinWidth != ps->BinWidth) {
	 sprintf(szBuf, "Original binwidth = %2.2f s, new binwidth = %3.1f s - continue?", ps->BaseBinWidth, ps->BinWidth);
	 result = MessageBox(hwnd, szBuf, "Binwidth change", MB_YESNO);
	 if(result == IDNO) return(NO_ERRORS); 
 }
 result = SavePHCFile(hwnd, ps,  TRUE);
 return(NO_ERRORS);
}


/******************************************************************************************/

// HELPER FUNCTIONS FOR THE ABOVE CODE

/*******************************************************************************************
  Removes the file extension from a filename (i.e. removes .xxx from the string)
 ****************************************************************************************/
char *remove_fext(char *str)
{
  int str_length = (strlen(str) - 1);

  while (1) {
    if (str[str_length] == '.') {
	  str[str_length] = '\0';
	  return(str);
	  }
    if (str_length==0) return (str);
    --str_length;
    }
}

/*******************************************************************************************
  Creates a filename based on the date
 ****************************************************************************************/
char *CreateDateFilename(char *fext)
{
 char Filename[10];
 static char szBuf[30]; 
 
 _strdate(Filename);
 Filename[2] = Filename[3];
 Filename[3] = Filename[4];
 Filename[4] = Filename[6];
 Filename[5] = Filename[7];
 Filename[6] = '\0';
 sprintf(szBuf,"%s.%s", Filename, fext);
 return(szBuf);
}

/*******************************************************************************************
  Initializes the OPENFILENAME struct for loading data files
 ****************************************************************************************/
char *GetOpenFile (HWND hwnd, char * szFilter, PHOTON_STREAM *ps)   // 3-17-09
{
 static OPENFILENAME ofn ;
 static char szFileName[256];

 char szFilePath[256];     // 3-17-09
 szFilePath[0] = 0;        // 3-17-09

 szFileName[0] = '\0';

 ofn.lStructSize       = sizeof (OPENFILENAME) ;
 ofn.hwndOwner         = hwnd ;
 ofn.hInstance         = NULL ;
 ofn.lpstrFilter       = szFilter ;
 ofn.lpstrCustomFilter = NULL ;
 ofn.nMaxCustFilter    = 0 ;
 ofn.nFilterIndex      = 0 ;
 ofn.lpstrFile         = szFilePath ;   // 3-17-09         
 ofn.nMaxFile          = MAX_PATH ;
 ofn.lpstrFileTitle    = szFileName ;          
 ofn.nMaxFileTitle     = MAX_PATH ;
 ofn.lpstrInitialDir   = NULL ;
 ofn.lpstrTitle        = NULL ;
 ofn.Flags             = OFN_HIDEREADONLY ;
 ofn.nFileOffset       = 0 ;
 ofn.nFileExtension    = 0 ;
 ofn.lpstrDefExt       = NULL;
 ofn.lCustData         = 0L ;
 ofn.lpfnHook          = NULL ;
 ofn.lpTemplateName    = NULL ;

 if(GetOpenFileName(&ofn)) {

     // Store full path filename to PHOTON_STREAM struct (3-17-09)
     memcpy( ps->szFilePath, szFilePath, 256 ); 

     // Store base filename to PHOTON_STREAM struct (3-17-09)
     memcpy( ps->szFilename, szFileName, 256 ); 

     return(szFileName) ;
 }

 return(szFileName);
}


/*******************************************************************************************
  Initializes the OPENFILENAME struct for saving data files
 ****************************************************************************************/
char *GetSaveFile(HWND hwnd, char *szFilename, char *szFilter, char *szExt)
{
 static char szFileName[256];
 static OPENFILENAME ofn ;  
 
 ofn.lStructSize       = sizeof (OPENFILENAME) ;
 ofn.hwndOwner         = hwnd ;
 ofn.hInstance         = NULL ;
 ofn.lpstrFilter       = szFilter ;
 ofn.lpstrCustomFilter = NULL ;
 ofn.nMaxCustFilter    = 0 ;
 ofn.nFilterIndex      = 0 ;
 ofn.lpstrFile         = szFilename;          
 ofn.nMaxFile          = MAX_PATH ;
 ofn.lpstrFileTitle    = szFileName ;          
 ofn.nMaxFileTitle     = MAX_PATH ;
 ofn.lpstrInitialDir   = NULL ;
 ofn.lpstrTitle        = NULL ;
 ofn.Flags             = OFN_OVERWRITEPROMPT ;
 ofn.nFileOffset       = 0 ;
 ofn.nFileExtension    = 0 ;
 ofn.lpstrDefExt       = szExt ;
 ofn.lCustData         = 0L ;
 ofn.lpfnHook          = NULL ;
 ofn.lpTemplateName    = NULL ;
 if(GetSaveFileName(&ofn)) return(szFileName) ;
 szFileName[0] = '\0';
 return(szFileName);
}

