/***********************************************************************
 BURSTPLOT.C - QC-based plotting routines used for displaying burst, 
               PeakHist, FCS and PCH results
 **********************************************************************/
#include <windows.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "burst.h"

/**************************************************************************************
 Displays the burst histograms
 ************************************************************************************/
void DisplayBurstHistograms(BURST *burst)
{
  ScaleHistArray(0, burst);
  CalcHistogram(pd[0].x, pd[0].y, pd[0].n, burst->Burst, 0, burst->numbursts);  
  GetMinMax(pd[0].y, pd[0].n, &pd[0].ymin, &pd[0].ymax);
  PlotHistogram(0);
  
  ScaleHistArray(1, burst);
  CalcHistogram(pd[1].x, pd[1].y, pd[1].n, burst->Width, 0, burst->numbursts);  
  GetMinMax(pd[1].y, pd[1].n, &pd[1].ymin, &pd[1].ymax);
  PlotHistogram(1);

  ScaleHistArray(2, burst);
  CalcHistogram(pd[2].x, pd[2].y, pd[2].n, burst->Interval, 0, burst->numbursts);  
  GetMinMax(pd[2].y, pd[2].n, &pd[2].ymin, &pd[2].ymax);
  PlotHistogram(2);
}


/**************************************************************************
 Sets up scaling values for the histogram plots
 pd[0 to 2] is a global plotdata structure used to display histogram data
*************************************************************************/
void ScaleHistArray(int win, BURST *burst)
{
  int i, n; 
  double max, min, mean, sd, Binsize;

  n = burst->numbursts - 1; 

  // find mean, max etc of data array
  switch(win) {
 	     case BURSTSIZE:  // number of photons per burst
		      CalcMean(burst->Burst, 0, n, &mean, &sd, &min, &max);
              burst->BurstMean = mean;
              burst->BurstSD = sd;
              burst->BurstMax = max;
			  burst->BurstMin = min;
		      break;

         case WIDTH:
		      CalcMean(burst->Width, 0, n, &mean, &sd, &min, &max);
              burst->WidthMean = mean;
              burst->WidthSD = sd;
              burst->WidthMax = max;
			  burst->WidthMin = min;
		      break;  
   
         case INTERVAL:
		      CalcMean(burst->Interval, 0, n, &mean, &sd, &min, &max);
              burst->IntervalMean = mean;
              burst->IntervalSD = sd;
              burst->IntervalMax = max;
			  burst->IntervalMin = min;
		      break;  
	  }

  if(init.bHistAutoScale) {               // autoscale binwidth
	  pd[win].n = init.NumHistBins;       // user selected number of bins
      pd[win].xmin = min;
	  pd[win].xmax = (max > 0.0)? max : 1.0; 
      pd[win].xstep = pd[win].xmax / (double) pd[win].n;
  }
  else {  //  use user-supplied values for histogram plot
       switch(win) {
 	     case BURSTSIZE:
		      Binsize = init.BurstBinsize;      // number of photons
		      break;

         case WIDTH:
		      Binsize = init.WidthBinsize;      // microsec
              if(Binsize = 0.0) Binsize = (double) init.BinWidth;
		      break;  
   
         case INTERVAL:
		      Binsize = init.IntervalBinsize;   // millisec
		      break;  
	  }
      pd[win].n = init.NumHistBins;
      pd[win].xmin = 0;
	  pd[win].xmax = pd[win].n * Binsize;   
      pd[win].xstep = pd[win].xmax / (double) pd[win].n;
  }
  
  // width can only vary as a multiple of the binwidth
  if((win == WIDTH) && (pd[win].xstep < init.BinWidth))  pd[win].xstep = (double) init.BinWidth;
  for(i=0;i<pd[win].n;i++ ) pd[win].x[i] = pd[win].xstep * (double) i;    
}


/***********************************************************************
 Calculates a histogram of lpSort based on the x values
 Size is the size of the x and y ptrs
 n is the number of points in array
 ************************************************************************/
void CalcHistogram(double *x, double *y, DWORD n, double *lpSort, DWORD start, DWORD end)
{
 DWORD i, j;
 double testval;
	 
 ZeroMemory((PVOID) y, n * sizeof(double));
 
 i = start;
 while(i < end) {   // i goes from 0 to numbursts
  	   j = 0;           // index of x
	   while(j < n-1) {
		    testval = lpSort[i];
			if((testval >= x[j]) && (testval < x[j+1])) {
               y[j] += 1.0;   
 	           break;   // leave while
			}
	        else 
               j++;  // look at next xval
	   }
	   i++;
   }  // end of i < end while
}


/*******************************************************************************************************************
  Plot the histogram data --  win values are defined in burst.h
 ******************************************************************************************************************/
void PlotHistogram(int win)
{
 char szBuf[12];

 sprintf(szBuf, "hist%d", win);
 WGSetLineStyle(pd[win].pGraph, pd[win].pGraph->hdc, PS_SOLID, 2, C_BLACK);
 pd[win].hData = WGDefineDataSetPtr (szBuf, pd[win].x, pd[win].y, pd[win].n);
 WGReconnectDataSet(pd[win].pGraph, pd[win].hPlot, pd[win].hData);
 WGReAxes(pd[win].pGraph, pd[win].xmin, pd[win].ymin, pd[win].xmax, pd[win].ymax, AS_EXACT, AS_ROUNDCLOSE, INTF_ZERO, pd[win].hAxisX, pd[win].hAxisY);
 WGChangeBarWidth (pd[win].pGraph, pd[win].hPlot, pd[win].xstep);
 WGRedrawGraph (pd[win].pGraph, TRUE);
}



/*******************************************************************************************************************
  Initialize QC graph page for main photon stream window
 ******************************************************************************************************************/
void  CALLBACK InitCountsPage(PPAGE_DEF pPageDesc)
{
 cd.pGraph = WGCreateGraph (pPageDesc, InitCountsGraph, 0.0, 0.0, 1.0, 1.0, C_WHITE, C_BLACK, 2); 
}

/*******************************************************************************************************************
  Initialize QC counts graph
 ******************************************************************************************************************/
void  CALLBACK InitCountsGraph(PGRAPH_DEF pGrDesc, HDC hdc)
{
 int i; 

 cd.pGraph = pGrDesc;
 cd.n = QC_SIZE;
 WGSetTextByName (C_BLACK, "Arial", 12, TEXT_NORMAL);
 WGSetPlotArea(pGrDesc, hdc, 0.05, 0.1, 0.98, 0.70, C_WHITE);
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 1, C_BLACK);
 cd.xmin = 0.0;
 cd.xmax = 100.0;
 cd.ymin = 0.0;
 cd.ymax = 100.0;
 cd.hData1 = WGDefineDataSetPtr("count1", cd.x, cd.y1, cd.n);
 cd.hData2 = WGDefineDataSetPtr("count2", cd.x, cd.y2, cd.n);
 WGAxes(pGrDesc, hdc, cd.xmin, cd.ymin, cd.xmax, cd.ymax, AS_ROUNDCLOSE, AS_EXACT, INTF_MIN, &cd.hAxisX, &cd.hAxisY, NULL, NULL);
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 2, C_BLACK);
 WGTitleAxis(pGrDesc, hdc, cd.hAxisX, POS_BELOW, "Time (s)");
 WGTitleAxis(pGrDesc, hdc, cd.hAxisY, POS_LEFT, "Photons");
 cd.hCur1 = WGSetDataCursor(pGrDesc, hdc, DC_VLINE, cd.hAxisX, cd.hAxisY, 0, 2, PS_SOLID, C_RED); 
 cd.hCur2 = WGSetDataCursor(pGrDesc, hdc, DC_VLINE, cd.hAxisX, cd.hAxisY, 0, 2, PS_SOLID, C_BLUE); 
 cd.hBurstMarker = WGSetDataCursor(pGrDesc, hdc, DC_CROSS, cd.hAxisX, cd.hAxisY, 10, 3, PS_SOLID, C_BLACK); 
 for(i=0;i<MAXBUR;i++) {
	 cd.hCurStart[i] = WGSetDataCursor(cd.pGraph, hdc, DC_VLINE, cd.hAxisX, cd.hAxisY, 0, 2, PS_SOLID, C_LIGHTGREEN); 
     cd.hCurEnd[i] = WGSetDataCursor(cd.pGraph, hdc, DC_VLINE, cd.hAxisX, cd.hAxisY, 0, 2, PS_SOLID, C_LIGHTRED); 
 }
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 2, C_BLACK);
 cd.hPlot1 = WGLinePlot(pGrDesc, hdc, cd.hData1, FALSE, FALSE);	
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 2, C_RED);
 cd.hPlot2 = WGLinePlot(pGrDesc, hdc, cd.hData2, FALSE, FALSE);	
}


/*******************************************************************************************************************
  Update the counts window
 ******************************************************************************************************************/
void DisplayData(void)
{
 DWORD i, j, k, start, end, dwVal1, dwVal2, n, num;
 double ymin, ymax, ymax1, ymax2, xmin, xmax, min;
   
 cd.cur1 = 0;
 cd.cur2 = cd.n- 1;
  
 n = (DWORD) cd.n;
 start = cd.SegNum * cd.BinsPerSegPt * n;
 end = (cd.SegNum + 1) * cd.BinsPerSegPt * n;
 if(end > cd.nbins) return; 
 
 num = cd.BinsPerSegPt; 

 switch (cd.Displaymode) {
     case IDC_CD_AVE:
	 case IDC_CD_SUM:
		 k = start;   // index of counts array
		 for(i=0; i<n;i++) {
			 dwVal1 = dwVal2 = 0;
			 cd.y1[i] = cd.y2[i] = 0.0;
			 for(j=0;j<num;j++) {
				 if(flags.bSet[0]) dwVal1 += (DWORD) ps1.counts[k];
				 if(flags.bSet[1]) dwVal2 += (DWORD) ps2.counts[k];
				 k++;
			 }
			 if(cd.Displaymode == IDC_CD_AVE) { 
				  dwVal1 /= num; 
				  dwVal2 /= num;
			 }
			 if(flags.bSet[0]) cd.y1[i] = (double) dwVal1;
 			 if(flags.bSet[1]) cd.y2[i] = (double) dwVal2;
		 }
	     break;
	 
	 case IDC_CD_SINGLE:
		 k = start;   // index of counts array
		 for(i=0;i<n;i++) {
 			 cd.y1[i] = cd.y2[i] = 0.0;
			 if(flags.bSet[0]) cd.y1[i] = (double) ps1.counts[k];
			 if(flags.bSet[1]) cd.y2[i] = (double) ps2.counts[k];
			 k += num;
		 }
		 break;
 }  // end of Displaymode switch

 // load the x array (time)
 for(i=0;i<n;i++) {
	cd.x[i] = (i * num + start) * cd.binwidth * US_TO_SEC;
 }

 xmin = cd.x[0];
 xmax = cd.x[cd.n - 1];
 ymin = 0.0;
 WGSetLineStyle(cd.pGraph, cd.pGraph->hdc, PS_SOLID, 2, C_BLACK);
 WGSetTextByName (C_BLACK, "Arial", 12, TEXT_NORMAL);
 if(init.bAutoScale) {
	 if(flags.bSet[0]) GetMinMax(cd.y1, cd.n, &ymax1, &min);
     if(flags.bSet[1]) GetMinMax(cd.y2, cd.n, &ymax2, &min);
     ymax = (ymax1 > ymax2) ? ymax1 : ymax2;
     WGRoundAxis3(cd.pGraph, cd.hAxisY, AS_ROUNDCLOSE, &ymin, &ymax, NULL, TRUE);
 }
 else {
	 ymax = (double) (ps1.max > ps2.max) ? ps1.max : ps2.max;   
 }
 cd.ymax = ymax;
 WGReAxes(cd.pGraph, xmin, ymin, xmax, ymax, AS_EXACT, AS_ROUNDCLOSE, INTF_ZERO, cd.hAxisX, cd.hAxisY);
 WGChangeAxisIntercept(cd.pGraph, cd.hAxisY,  xmin, TRUE);

 if(flags.bResultsReady[active_ps] && cd.bShowBurstMarkers) MarkBursts(active_ps);
}


/*******************************************************************************************************************
  Initialize QC graph page for burst and FCS results 
 ******************************************************************************************************************/
void CALLBACK InitResultsPage(PPAGE_DEF pPageDesc)
{
 WGCreateGraph (pPageDesc, InitPhotonsGraph, 0.0, 0.0, 0.37, 0.5, C_WHITE, C_BLACK, 2); 
 WGCreateGraph (pPageDesc, InitBurstWidthGraph, 0.0, 0.5, 0.37, 1.0, C_WHITE, C_BLACK, 2); 
 WGCreateGraph (pPageDesc, InitIntervalGraph, 0.37, 0.0, 0.74, 0.5, C_WHITE, C_BLACK, 2); 
 WGCreateGraph (pPageDesc, InitFCSGraph, 0.37, 0.5, 0.74, 1.0, C_WHITE, C_BLACK, 2); 
 pText = WGCreateGraph (pPageDesc, InitDrawText, 0.74, 0.0, 1.0, 1.0, C_WHITE, C_BLACK, 2); 
 WGToggleSelectors(pPageDesc, FALSE); 
}


/*******************************************************************************************************************
  Initialize QC graph photons/burst
 ******************************************************************************************************************/
void  CALLBACK InitPhotonsGraph(PGRAPH_DEF pGrDesc, HDC hdc)
{
 pd[0].pGraph = pGrDesc;
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 1, C_BLACK);
 WGSetTextByName (C_BLACK, "Arial", 12, TEXT_NORMAL);
 WGSetPlotArea(pGrDesc, hdc, 0.12, 0.1, 0.9, 0.75, C_WHITE);
 pd[0].n = 2;
 pd[0].xmin = 0.0;
 pd[0].xmax = 100.0;
 pd[0].ymin = 0.0;
 pd[0].ymax = 100.0;
 pd[0].hData = WGDefineDataSetPtr ("burst", pd[0].x, pd[0].y, pd[0].n);
 WGAxes(pGrDesc, hdc, pd[0].xmin, pd[0].ymin, pd[0].xmax, pd[0].ymax, AS_ROUNDCLOSE, AS_ROUNDCLOSE, INTF_MIN, &pd[0].hAxisX, &pd[0].hAxisY, NULL, NULL);
 pd[0].hPlot = WGBargraph(pGrDesc, hdc, pd[0].hData, pd[0].xstep, POS_MIDDLE, LT_VBAR, RGB(0,0,0), -1, -1);
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 1, C_BLACK);
 WGTitleAxis(pGrDesc, hdc, pd[0].hAxisX, POS_BELOW, "Photons/Burst");
 WGTitleAxis(pGrDesc, hdc, pd[0].hAxisY, POS_LEFT, "Frequency");
 }

/*******************************************************************************************************************
  Initialize QC graph for burst width
 ******************************************************************************************************************/
void  CALLBACK InitBurstWidthGraph(PGRAPH_DEF pGrDesc, HDC hdc)
{
 pd[1].pGraph = pGrDesc;
 WGSetTextByName (C_BLACK, "Arial", 12, TEXT_NORMAL);
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 1, C_BLACK);
 WGSetPlotArea(pGrDesc, hdc, 0.12, 0.1, 0.9, 0.75, C_WHITE);
 pd[1].n = 2;
 pd[1].xmin = 0.0;
 pd[1].xmax = 100.0;
 pd[1].ymin = 0.0;
 pd[1].ymax = 100.0;
 pd[1].hData = WGDefineDataSetPtr ("width", pd[1].x, pd[1].y, pd[1].n );
 WGAxes(pGrDesc, hdc, pd[1].xmin, pd[1].ymin, pd[1].xmax, pd[1].ymax, AS_ROUNDCLOSE, AS_ROUNDCLOSE, INTF_MIN, &pd[1].hAxisX, &pd[1].hAxisY, NULL, NULL);
 pd[1].hPlot = WGBargraph(pGrDesc, hdc, pd[1].hData, 0.5, POS_MIDDLE, LT_VBAR, C_BLACK, -1, -1);
 WGTitleAxis(pGrDesc, hdc, pd[1].hAxisX, POS_BELOW, "Burst width (s)");
 WGTitleAxis(pGrDesc, hdc, pd[1].hAxisY, POS_LEFT, "Frequency");
}

/*******************************************************************************************************************
  Initialize QC graph for burst interval
 ******************************************************************************************************************/
void  CALLBACK InitIntervalGraph(PGRAPH_DEF pGrDesc, HDC hdc)
{
 pd[2].pGraph = pGrDesc;
 WGSetPlotArea(pGrDesc, hdc, 0.12, 0.1, 0.9, 0.75, C_WHITE);
 pd[2].n = 2;
 pd[2].xmin = 0.0;
 pd[2].xmax = 100.0;
 pd[2].ymin = 0.0;
 pd[2].ymax = 100.0;
 pd[2].hData = WGDefineDataSetPtr ("interval", pd[2].x, pd[2].y, pd[2].n);
 WGAxes(pGrDesc, hdc, pd[2].xmin, pd[2].ymin, pd[2].xmax, pd[2].ymax, AS_ROUNDCLOSE, AS_ROUNDCLOSE, INTF_MIN, &pd[2].hAxisX, &pd[2].hAxisY, NULL, NULL);
 WGSetTextByName (C_BLACK, "Arial", 12, TEXT_NORMAL);
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 2, C_BLACK);
 pd[2].hPlot = WGBargraph(pGrDesc, hdc, pd[2].hData, 0.5, POS_MIDDLE, LT_VBAR, C_BLACK, -1, -1);
 WGTitleAxis(pGrDesc, hdc, pd[2].hAxisX, POS_BELOW, "Interval (ms)");
 WGTitleAxis(pGrDesc, hdc, pd[2].hAxisY, POS_LEFT, "Frequency");
}


/******************************************************************************************
 Initialize the QC plot window for displaying FCS curves
 *******************************************************************************************/
void  CALLBACK InitFCSGraph(PGRAPH_DEF pGrDesc, HDC hdc)
{
 pdFCS.pGraph = pGrDesc;
 WGSetPlotArea(pGrDesc, hdc, 0.12, 0.1, 0.9, 0.75, C_WHITE);
 pdFCS.n = 2;
 pdFCS.x[0] = pdFCS.xmin = 1.0;
 pdFCS.x[1] = pdFCS.xmax = 10000.0;
 pdFCS.y[0] = pdFCS.ymin = 0.0;
 pdFCS.y[1] = pdFCS.ymax = 0.0;
 pdFCS.hData = WGDefineDataSetPtr("correlation", pdFCS.x, pdFCS.y, pdFCS.n);
 WGSetAxesScale (pGrDesc, ST_LOG, ST_LINEAR);
 WGAxes(pGrDesc, hdc, pdFCS.xmin, pdFCS.ymin, pdFCS.xmax, pdFCS.ymax, AS_ROUNDCLOSE, AS_ROUNDCLOSE, INTF_MIN, &pdFCS.hAxisX, &pdFCS.hAxisY, NULL, NULL);
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 1, C_BLACK);
 WGSetTextByName (C_BLACK, "Arial", 12, TEXT_NORMAL);
 pdFCS.hPlot = WGLinePlot (pGrDesc, hdc, pdFCS.hData, FALSE, FALSE);
 WGTitleAxis(pGrDesc, hdc, pdFCS.hAxisX, POS_BELOW, "Lag time (ms)");
 WGTitleAxis(pGrDesc, hdc, pdFCS.hAxisY, POS_LEFT,  "G(t)");
 strcpy(pdFCS.szPlotTitle, "Correlation"); 
 pdFCS.hTitle = WGTitleGraph(pGrDesc, hdc, pdFCS.szPlotTitle);
}


/*******************************************************************************************************************
  Initialize QC text page for displaying parameters
 ******************************************************************************************************************/
void  CALLBACK InitDrawText(PGRAPH_DEF pGrDesc, HDC hdc)
{
 int i;
 double x, y, yoffset, yinc;
 char szBuf[80];
 char szBlank[] = "XXXXXXXXXXXXXXXXXXX";
 char szLabel[NUMTEXT][50] = {"Filename:", "Counts/sec:", "Background counts:",
	                          "Number of bursts:", "Mean photons/burst:", 
							  "Mean peak value:", "Mean burst width:", 
							  "Mean burst interval:"};
 
 WGSetTextByName (C_BLACK, "Arial", 11, TEXT_NORMAL);
 WGSetPlotArea(pGrDesc, hdc, 0.08, 0.1, 0.98, 0.70, C_WHITE);
 WGTextNorm(pGrDesc, hdc, "Analysis Results", 0.03, 0.08, TA_LEFT|TA_TOP, TEXT_NORMAL); 
 
 yinc = 0.05;
 yoffset = 0.2;
 x = 0.05;
 for(i=0;i<NUMTEXT; i++) {
	 sprintf(szBuf, "%s", szLabel[i]);
     y = yoffset + (yinc * (double) i); 
	 WGTextNorm(pGrDesc, hdc, szBuf, x, y, TA_LEFT|TA_TOP, TEXT_NORMAL);
 }

 // create and hide text objects for fit results
 x = 0.50;
 for(i=0;i<NUMTEXT; i++) {
	 y = yoffset + yinc * (double) i; 
	 hText[i] = WGTextNorm(pGrDesc, hdc, szBlank, x, y, TA_LEFT|TA_TOP, TEXT_NORMAL);
	 WGShowObject(pGrDesc, hText[i], FALSE);
 }
}

/*******************************************************************************************************************
  Update the text in the parameter display window
 ******************************************************************************************************************/
void UpDateText(PHOTON_STREAM *ps, BURST *burst) 
{
 int i; 
 char szBuf[80];
 
 if(ps->counts == NULL) {
	 for(i=0; i<NUMTEXT; i++)  WGShowObject(pText, hText[i], FALSE);
	 return;
 }

 if(flags.bSet[active_ps]) {
    sprintf(szBuf, "%s" , ps->szFilename);
	WGChangeString(pText,  hText[0], szBuf, 0);
    WGShowObject(pText, hText[0], TRUE);
 
    sprintf(szBuf, "%4.3f KHz" , ps->CPS);
    WGChangeString(pText,  hText[1], szBuf, 0);
    WGShowObject(pText, hText[1], TRUE);
 
    sprintf(szBuf, "%3.1f  %3.1f", ps->BkMean, ps->BkSD);
    WGChangeString(pText,  hText[2], szBuf, 0);
    WGShowObject(pText, hText[2], TRUE);
 }
 
 if(flags.bResultsReady[active_ps]) {
    sprintf(szBuf, "%u", burst->numbursts);
    WGChangeString(pText,  hText[3], szBuf, 0);
    WGShowObject(pText, hText[3], TRUE);

    sprintf(szBuf, "%5.2f  %5.2f photons", burst->BurstMean, burst->BurstSD); 
    WGChangeString(pText,  hText[4], szBuf, 0);
    WGShowObject(pText, hText[4], TRUE);

    sprintf(szBuf, "%4.2f  %3.2f photons", burst->PeakMean, burst->PeakSD); 
    WGChangeString(pText,  hText[5], szBuf, 0);
    WGShowObject(pText, hText[5], TRUE);

    sprintf(szBuf, "%4.2f  %3.2f s", burst->WidthMean, burst->WidthSD); 
    WGChangeString(pText,  hText[6], szBuf, 0);
    WGShowObject(pText, hText[6], TRUE);
 
    sprintf(szBuf, "%4.2f  %3.2f sec", burst->IntervalMean, burst->IntervalSD); 
    WGChangeString(pText,  hText[7], szBuf, 0);
    WGShowObject(pText, hText[7], TRUE);
 
 }
}




/*******************************************************************************************************************
  Plot the FCS curve 
 ******************************************************************************************************************/
void PlotFCSCurve(AUTOCORRELATION *fcs, char *szText)
{
 int i;

 pdFCS.n = fcs->ntau;
 for(i=0;i<pdFCS.n;i++) {
	 pdFCS.x[i] = fcs->ac[i].Lag;
	 pdFCS.y[i] = fcs->ac[i].Gn;
 }

 WGSetLineStyle(pdFCS.pGraph, pdFCS.pGraph->hdc, PS_SOLID, 2, C_BLACK);
 WGSetTextByName (C_BLACK, "Arial", 12, TEXT_NORMAL);
 pdFCS.hData = WGDefineDataSetPtr ("ac", pdFCS.x, pdFCS.y, pdFCS.n);
 WGReconnectDataSet(pdFCS.pGraph, pdFCS.hPlot, pdFCS.hData);
 pdFCS.xmin = pdFCS.x[0];
 pdFCS.xmax = pdFCS.x[pdFCS.n-1];   
 WGRoundAxis3(pdFCS.pGraph, pdFCS.hAxisX, AS_ROUNDCLOSE, &pdFCS.xmin, &pdFCS.xmax, NULL, TRUE);
 WGChangeAxisRange(pdFCS.pGraph, pdFCS.hAxisX, pdFCS.xmin, pdFCS.xmax, TRUE);
 WGGetMinMaxY(pdFCS.hData, &pdFCS.ymin, &pdFCS.ymax);
 pdFCS.ymin = 0.0;
 WGRoundAxis3(pdFCS.pGraph, pdFCS.hAxisY, AS_ROUNDCLOSE, &pdFCS.ymin, &pdFCS.ymax, NULL, TRUE);
 WGChangeAxisRange(pdFCS.pGraph, pdFCS.hAxisY, pdFCS.ymin, pdFCS.ymax, TRUE);
 WGShowObject(pdFCS.pGraph, pdFCS.hPlot, TRUE);
 strcpy(pdFCS.szPlotTitle, szText); 
 WGChangeString(pdFCS.pGraph,  pdFCS.hTitle, szText, 0);
 WGShowObject(pdFCS.pGraph, pdFCS.hTitle, TRUE);
 WGRedrawGraph(pdFCS.pGraph, TRUE);
}

/*******************************************************************************************************************
  Clears the histogram data sets and plots
 ******************************************************************************************************************/
void ClearGraphs(void)
{
 int i;
 char szName[10];

 for(i=0;i<3;i++) {
	  pd[i].n = 2;
      pd[i].xmin = 0.0;
      pd[i].xmax = 100.0;
      pd[i].ymin = 0.0;
      pd[i].ymax = 100.0;
   	  sprintf(szName, "temp%d", i);
	  pd[i].hData = WGDefineDataSetPtr (szName, pd[0].x, pd[0].y, pd[0].n);
      WGReAxes(pd[i].pGraph, pd[0].xmin, pd[0].ymin, pd[0].xmax, pd[0].ymax, AS_ROUNDCLOSE, AS_ROUNDCLOSE, INTF_MIN, pd[0].hAxisX, pd[0].hAxisY);
	  WGReconnectDataSet(pd[i].pGraph, pd[i].hPlot, pd[i].hData);
 }
}

 


/******************************************************************************************
 Initialize the QC plot window page for displaying PCH curves
 *******************************************************************************************/
void  CALLBACK InitPCHGraph(PPAGE_DEF pPageDesc)
{
 WGCreateGraph (pPageDesc, InitPCHPlot, 0.0, 0.0, 1.0, 1.0, C_WHITE, C_BLACK, 2); 
}

/******************************************************************************************
 Disply the PCH curve
 *******************************************************************************************/
void  CALLBACK InitPCHPlot(PGRAPH_DEF pGrDesc, HDC hdc)
{
 double *x, *y, ymax, xmax;
 int    i, n; 
 HDATA  hData;        
 HGOBJ  hAxisY;
 HGOBJ  hAxisX;

 n = pch.n;
 xmax = pch.xmax;
 ymax = pch.ymax;
 x = (double *) calloc(n, sizeof(double));
 y = (double *) calloc(n, sizeof(double));
 for(i=0;i<n;i++) {
	 x[i] = pch.x[i];
	 y[i] = pch.y[i];
}
 
 WGToggleSelectors (pGrDesc->pPageDesc, FALSE) ;
 WGSetTextByName (C_BLACK, "Arial", 12, TEXT_NORMAL);
 WGSetPlotArea(pGrDesc, hdc, 0.2, 0.1, 0.9, 0.70, C_WHITE);
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 1, C_BLACK);
 hData = WGDefineDataSetPtr("pch1", x, y, n);
 WGAxes(pGrDesc, hdc, 0.0, 0.0, xmax, ymax, AS_ROUNDCLOSE, AS_EXACT, INTF_MIN, &hAxisX, &hAxisY, NULL, NULL);
 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 2, C_BLACK);
 WGTitleAxis(pGrDesc, hdc, hAxisX, POS_BELOW, "Photons/bin");
 WGTitleAxis(pGrDesc, hdc, hAxisY, POS_LEFT, "Freq.");
 if(active_ps == 0)
	 WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 2, C_BLACK);
 else
     WGSetLineStyle(pGrDesc, hdc, PS_SOLID, 2, C_RED);

 hPCHPlot = WGLinePlot(pGrDesc, hdc, hData, FALSE, FALSE);	
 }


/*********************************************************************************************************
 Converts counts plot x index value to ps->counts x index value
  *********************************************************************************************************/
DWORD GetPSIndexFromCountsIndex(int CountsXindex) 
{
  return((cd.SegNum * cd.BinsPerSegPt * (DWORD) cd.n) + CountsXindex * cd.BinsPerSegPt);
}

/*********************************************************************************************************
 Converts ps->counts x index value to counts plot x index value 
  *********************************************************************************************************/
DWORD GetCountsIndexFromPSIndex(int PSXindex) 
{
  return((PSXindex - (cd.SegNum * cd.BinsPerSegPt * (DWORD) cd.n))/cd.BinsPerSegPt);
}


/*********************************************************************************************************
  Draws a QC cursors at the start and stop of each burst shown in the counts plot
  *********************************************************************************************************/
void MarkBursts(int set)
{
 DWORD i, j, nb, start, end, stopindex;
 int index1, index2; 
 BURST *burst = GETBURST(set);

 nb = burst->numbursts; 

 // find the indices of the burst Start and End arrays that lie with the displayed segment
 start = GetPSIndexFromCountsIndex(0);
 end = GetPSIndexFromCountsIndex(cd.n-1); 
 stopindex = MAXBUR;
 i = j = 0;
 while(i < nb) {
	 if(burst->Start[i] >= start && burst->Start[i] <= end && (j < stopindex)) {
	     index1 = (int) GetCountsIndexFromPSIndex(burst->Start[i]); 
		 index2 = (int) GetCountsIndexFromPSIndex(burst->End[i]); 
	     WGMoveDataCursor(cd.pGraph, cd.hCurStart[j], cd.x[index1], cd.ymax, TRUE);
	     WGMoveDataCursor(cd.pGraph, cd.hCurEnd[j], cd.x[index2], cd.ymax, TRUE);
         cd.MarkerIndex[j] = index2 - ((index2 - index1)/2);
		 cd.BurstIndex[j] = i; 
		 j++; 
	 }
	 i++; 
 }
 cd.nMarkersInSegment =  j-1; 
 cd.Marker = -1;
 MoveBurstMarker(IDC_BURST_NEXT, set);
}

/*******************************************************************************************************************
  Moves the burst marker QC cursor to the next displayed burst
 ******************************************************************************************************************/
void MoveBurstMarker(int direction, int set)
{
  char szBuf[256];
  int i;
  BURST *burst = GETBURST(set);
  

  if(!flags.bResultsReady[set]) return; 
  switch(direction) {
	   case IDC_BURST_NEXT:
		   if(cd.Marker < cd.nMarkersInSegment) cd.Marker++;
		   break;
	   case IDC_BURST_BACK:
		   if(cd.Marker > 0) cd.Marker--;
		   break;
	}
  i = cd.BurstIndex[cd.Marker]; 
  WGMoveDataCursor(cd.pGraph, cd.hBurstMarker, cd.x[cd.MarkerIndex[cd.Marker]], cd.ymax, TRUE);
  sprintf(szBuf, "%3.5f sec", burst->Time[i]); 
  SendDlgItemMessage(hPSControls, IDC_BURSTNUM, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));
  sprintf(szBuf,"#%d: %u photons (Peak = %u)     Width = %u s", cd.BurstIndex[cd.Marker]+1, (DWORD) burst->Burst[i],
	     (DWORD) burst->Peak[i], (DWORD) burst->Width[i]);
  SendDlgItemMessage(hPSControls, IDC_BURSTINFO, WM_SETTEXT,sizeof(szBuf),(LPARAM)((LPSTR) szBuf));


}
